###################################################################
# BSVAR-subset.R
#
# Analysis of macropolity IRFs for monthly models
#
# Brandt and Freeman BVAR 2 paper -- APSA and PA
#
# Patrick Brandt
# pbrandt@utdallas.edu
#
# 11/13/2005 -- Initial version
# 12/05/2005 -- Revised to analyze new data files.
# 01/30/2006 -- Re-ran IRF plots to reflect changes in code
#               base.
# 02/13/2006 -- Added additional comments to clarify code
# 02/16/2006 -- Added new graph for the Macropolity responses
#               to aid in writing up the interpretation.
# 04/18/2006 -- Updated to use latest MSBVAR and updated model
#               sampling code.
# 09/07/2006 -- Revised to be compatible with latest model
#               runs.
# 10/05/2006 -- Change the responses presented in each figure to
#               reduce redundant presentation.
# 10/06/2006 -- Added computation of the cumulative responses and
#               their CI
# 12/04/2007 -- Revised to work with MSBVAR 0.3.1
# 04/02/2008 -- Added code to print out responses.
# 07/10/2008 -- Generate results for multiple priors.  Had to reorder
#               the computations to prevent buffer overflows.
# 07/17/2008 -- reimplemented cumulative responses
####################################################################

# Load libraries
library(MSBVAR)  # Brandt BVAR / VAR library

# Load data and functions
load("Data_Setup.RData")


######################################################################
# New function that uses a list of MCMC generated impulse responses
# and knows how to combine them into one set of graphs with smart
# coloring / line types.  Only implements the Sims-Zha2 and Percentile
# methods.
######################################################################

plot.mc.irf.svar.subset <- function(mc.impulse.list, num.eqn, num.shk,
                                    probs = c(0.16, 0.84),
                                    eqnnames = NULL,
                                    shknames = NULL,
                                    method="Percentile",
                                    component=1,
                                    ...)

{
    total <- num.eqn*num.shk
    m <- num.eqn
    nsteps <- dim(mc.impulse.list[[1]])[2]
    draws <- dim(mc.impulse.list[[1]])[1]

    # Now build a set of lists of arrays to store the irf summaries
    irf.storage <- array(0, c(nsteps, length(probs) + 1, total,
                              length(mc.impulse.list)))

    irf.ci <- array(0, c(nsteps, length(probs) + 1, total))

    if (method == "Percentile") {
    for (j in 1:length(mc.impulse.list))
    { if (total>1){
        for (i in 1:total) {
            irf.bands <- t(apply(mc.impulse.list[[j]][, , i], 2, quantile,
                                 probs))
            irf.mean <- apply(mc.impulse.list[[j]][, , i], 2, median)
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
        }
    } else { for (i in 1:total) {
        irf.bands <- t(apply(mc.impulse.list[[j]], 2, quantile,
                             probs))
        irf.mean <- apply(mc.impulse.list[[j]], 2, median)
        irf.ci[, , i] <- cbind(irf.bands, irf.mean)
    }
         }
      irf.storage[,,,j] <- irf.ci
  }
}

    if (method == "Sims-Zha2") {
      for (j in 1:length(mc.impulse.list))
        {
          for (i in 1:total) {
             if (total>1) {
                 decomp <- eigen(var(mc.impulse.list[[j]][, , i]),
                                 symmetric = T)
                 W <- decomp$vectors
                 lambda <- decomp$values
                 gammak <- mc.impulse.list[[j]][, , i] * (W[component, ])
                 gammak.quantiles <- t(apply(gammak, 2, quantile,
                                             probs = probs))
                 irf.mean <- apply(mc.impulse.list[[j]][, , i], 2,
                                   median)
                 # print eigen %
                 cat("Eigen %", (100*lambda/sum(lambda))[1], "\n")

             } else {
                 decomp <- eigen(var(mc.impulse.list[[j]]),
                 symmetric=T)
                          W <- decomp$vectors
                 lambda <- decomp$values
                 gammak <- mc.impulse.list[[j]] * (W[component, ])
                 gammak.quantiles <- t(apply(gammak, 2, quantile,
                                             probs = probs))
                 irf.mean <- apply(mc.impulse.list[[j]], 2, median)

                 cat("Eigen %", (100*lambda/sum(lambda))[1], "\n")

             }

            irf.bands <- irf.mean + gammak.quantiles
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
         }

          irf.storage[,,,j] <- irf.ci
      }
    }

    # Clean up
    rm(irf.ci)
    gc()

    # Now find the min-max values for the plots
    minmax <- array(0, c(num.eqn, 2, length(mc.impulse.list)))

##     for (j in 1:length(mc.impulse.list))
##     {
##         within.plots <- apply(irf.storage[,,,j], 3, range)
##         tmp <- (c(1:total)%%num.eqn)
##         tmp[tmp == 0] <- num.eqn
##         indices <- sort(tmp, index.return = T)$ix
##         dim(indices) <- c(num.shk, num.eqn)
##         for (i in 1:num.eqn) {
##             minmax[i,,j] <- range(within.plots[, indices[, i]])
##         }
##     }

    if(num.eqn>1)
    {
        for (j in 1:length(mc.impulse.list))
        {
            within.plots <- apply(irf.storage[,,,j], 3, range)
            tmp <- (c(1:total)%%num.eqn)
            tmp[tmp == 0] <- num.eqn
            indices <- sort(tmp, index.return = T)$ix
            dim(indices) <- c(num.shk, num.eqn)
            for (i in 1:num.eqn) {
                minmax[i,,j] <- range(within.plots[, indices[, i]])
            }
        }
    }else{ minmax <- array(range(irf.storage), c(num.eqn, 2,
    length(mc.impulse.list))) }

    response.mtx <- expand.grid(eqnnames, shknames)
    colnames(response.mtx) <- c("Equation", "Shock")

    # Now do the plotting
    j <- 1
    par(mfcol = c(num.eqn, num.shk), mai = c(0.25, 0.25, 0.15, 0.25),
        omi = c(0.35, 0.75, 1, 0.15), cex=0.9)
    for (i in 1:total) {
        lims <- ifelse((i-num.eqn)%%num.eqn==0, num.eqn,
                       (i-num.eqn)%%num.eqn)

        tmp <- matrix(irf.storage[, , i,], nrow=nsteps,
                      ncol=3*dim(irf.storage)[4])

        # Print the responses to STDOUT
        print(response.mtx[i,])
        print(tmp)

        ts.plot(tmp,
                gpars = list(xlab = "",
                lty=rep(seq(1:length(mc.impulse.list)), 1, each=3),
                ylab = "", #col=c(rep("black",3), rep("red", 3)),
                lwd=1,
                ylim = 1.1*range(minmax[lims,,])))
        abline(h = 0)
        if(i<=num.eqn)
        { mtext(eqnnames[i], side=2, line=3) }
        if((i-1)%%num.eqn==0)
        { mtext(shknames[j], side=3, line=2)
          j <- j+1

      }

    }
    mtext("Response in", side = 2, line = 3, outer = T)
    mtext("Shock to", side = 3, line = 3, outer = T)
}


# Subsets we want -- separate out the political and econ shocks --
# note that these are stored in column / outer array order for the IRF
# for the full 9 x 9.

sublist1 <- c(55:60, 64:69, 73:78)  # Responses to political shocks

sublist2 <- c(7:9, 16:18, 25:27, 34:36, 43:45, 52:54, 61:63, 70:72,
              79:81) # Responses to economic shocks

# Load in the impulse responses for the various priors and subset
# results

load("midrangeirfs.RData")
midrange.responses2politics <- midrange.irfs[,,sublist1]
midrange.responses2econ <- midrange.irfs[,,sublist2]
rm(midrange.irfs)
gc()

load("runningtallyirfs.RData")
running.tally.responses2politics <- running.tally.irfs[,,sublist1]
running.tally.responses2econ <- running.tally.irfs[,,sublist2]
rm(running.tally.irfs)
gc()

# Now make lists of the relevant IRFs to compare
irfs.responses2politics <- list(100*midrange.responses2politics,
                                100*running.tally.responses2politics)

irfs.responses2econ <- list(100*midrange.responses2econ,
                            100*running.tally.responses2econ)

rm(midrange.responses2politics, running.tally.responses2politics,
   midrange.responses2econ, running.tally.responses2econ)
gc(); gc();

# Generate sets with 68% error bands

# Informed prior responses to political shocks
pdf(file="responses2politics.pdf", width=5, height=8)

plot.mc.irf.svar.subset(irfs.responses2politics, num.eqn=6,
                        num.shk=3, method = c("Sims-Zha2"),
                        component = 1,
                        probs = c(0.16, 0.84),
                        eqnnames = c("Pcom", "M2", "R", "Y", "CPI",
                        "U"),
                        shknames = c("CS", "Approval", "MP"))

dev.off()
gc()

# Informed priors responses to economic shocks
pdf(file="responses2econ.pdf", width=9, height=4)

plot.mc.irf.svar.subset(irfs.responses2econ, num.eqn=3,
                        num.shk=9, method = c("Sims-Zha2"),
                        component = 1,
                        probs = c(0.16, 0.84),
                        eqnnames = c("CS", "Approval", "MP"),
                        shknames = c("Pcom", "M2", "R", "Y", "CPI",
                        "U", "CS", "A", "MP"))

dev.off()
gc()

# Generate all of the above sets with 90% error bands
pdf(file="responses2politics90.pdf", width=5, height=8)

plot.mc.irf.svar.subset(irfs.responses2politics, num.eqn=6,
                        num.shk=3, method = c("Sims-Zha2"),
                        component = 1,
                        probs = c(0.05, 0.95),
                        eqnnames = c("Pcom", "M2", "R", "Y", "CPI",
                        "U"),
                        shknames = c("CS", "Approval", "MP"))

dev.off()
gc()

pdf(file="responses2econ90.pdf", width=9, height=4)

plot.mc.irf.svar.subset(irfs.responses2econ, num.eqn=3,
                        num.shk=9, method = c("Sims-Zha2"),
                        component = 1,
                        probs = c(0.05, 0.95),
                        eqnnames = c("CS", "Approval", "MP"),
                        shknames = c("Pcom", "M2", "R", "Y", "CPI",
                        "U", "CS", "A", "MP"))

dev.off()


# Get the cumulative responses

# Vectors for the names!
eqn <- c("Pcom", "M2", "R", "Y", "CPI", "U")
shk <- c("CS", "Approval", "MP")

# Political responses
response.mtx <- expand.grid(eqn, shk)
colnames(response.mtx) <- c("Equation", "Shock")

prior <- c("Loose", "Tight")

for (i in 1:length(irfs.responses2politics))
{
    for (j in 1:dim(irfs.responses2politics[[i]])[3])
    {
        print(prior[i])
        print(response.mtx[j,])
        print(t(apply(apply(irfs.responses2politics[[i]][,,j], 1,
                            cumsum),
                      1, quantile,
                  probs=c(0.16, 0.5, 0.84))))
    }
}


# Economic responses
response.mtx <- expand.grid(shk, c(eqn,shk))
colnames(response.mtx) <- c("Equation", "Shock")

for (i in 1:length(irfs.responses2econ))
{
    for (j in 1:dim(irfs.responses2econ[[i]])[3])
    {
        print(prior[i])
        print(response.mtx[j,])
        print(t(apply(apply(irfs.responses2econ[[i]][,,j], 1,
                            cumsum),
                      1, quantile,
                  probs=c(0.16, 0.5, 0.84))))
    }
}


rm(irfs.responses2politics, irfs.responses2econ)
gc(); gc();


# Quit
q(save="no")
