# BSVARirf.R
#
# Samples impulse responses and graphs them for the Brandt and Freeman
# model of the macro-political-economy
#
# Patrick T. Brandt
#
# pbrandt@utdallas.edu
#
# 2006-08-16 : Original version
# 2006-08-21 : Added ML model
# 2007-12-04 : Revised to work with MSBVAR 0.3.1
########################################################################

# Load MSBVAR library
library(MSBVAR)

# Load setup
load("Data_Setup.RData")

# Set up equation names for labeling IRFs
vnames <- c("Pcom", "M2", "R",
            "Y", "CPI", "U",  "CS",
            "Approval", "MP")

#########################################
# Running Tally model IRFs
#########################################

# Load posterior sample
load("runningtallyposterior.RData")

# Normalize signs of shocks and just use the first chain.
ident <- matrix(0, 9, 9)
ident[which(running.tally.A0$ident==1)] <- signlist
running.tally.A0$N2 <- N2

# MCMC Sample the IRFS
running.tally.irfs <- mc.irf(running.tally.model, nsteps=48,
                             A0.posterior=running.tally.A0,
                             sign.list=diag(ident))

# Plot the results
pdf(file="runningtallyirfs.pdf", width=9, height=6)
plot(100*running.tally.irfs, method="Sims-Zha2", probs=c(0.16, 0.84),
     varnames=vnames)
dev.off()

save(running.tally.irfs, file="runningtallyirfs.RData", compress=T)
rm(running.tally.irfs, running.tally.A0)
gc()


#########################################
# Midrange model IRFs
#########################################

# Load posterior sample
load("midrangeposterior.RData")

# Normalize signs of shocks and just use the first chain.
ident <- matrix(0, 9, 9)
ident[which(midrange.A0$ident==1)] <- signlist
midrange.A0$N2 <- N2

# MCMC Sample the IRFS
midrange.irfs <- mc.irf(midrange.model, nsteps=48,
                        A0.posterior=midrange.A0,
                        sign.list=diag(ident))

# Plot the results
pdf(file="midrangeirfs.pdf", width=9, height=6)
plot(100*midrange.irfs, method="Sims-Zha2", probs=c(0.16, 0.84),
     varnames=vnames)
dev.off()

save(midrange.irfs, file="midrangeirfs.RData", compress=T)
rm(midrange.irfs, midrange.A0)
gc()


#########################################
# Best model IRFs
#########################################

# Load posterior sample
load("bestposterior.RData")

# Normalize signs of shocks and just use the first chain.
ident <- matrix(0, 9, 9)
ident[which(best.A0$ident==1)] <- signlist
best.A0$N2 <- N2

# MCMC Sample the IRFS
best.irfs <- mc.irf(best.model, nsteps=48,
                    A0.posterior=best.A0,
                    sign.list=diag(ident))

# Plot the results
pdf(file="bestirfs.pdf", width=9, height=6)
plot(100*best.irfs, method="Sims-Zha2", probs=c(0.16, 0.84),
     varnames=vnames)
dev.off()

save(best.irfs, file="bestirfs.RData", compress=T)
rm(best.irfs, best.A0)
gc()


#########################################
# Best ML model IRFs
#########################################

# Load posterior sample
load("bestMLposterior.RData")

# Normalize signs of shocks and just use the first chain.
ident <- matrix(0, 9, 9)
ident[which(bestML.A0$ident==1)] <- signlist
bestML.A0$N2 <- N2

# MCMC Sample the IRFS
bestML.irfs <- mc.irf(bestML.model, nsteps=48,
                      A0.posterior=bestML.A0,
                      sign.list=diag(ident))

# Plot the results
pdf(file="bestMLirfs.pdf", width=9, height=6)
plot(100*bestML.irfs, method="Sims-Zha2", probs=c(0.16, 0.84),
     varnames=vnames)
dev.off()

save(bestML.irfs, file="bestMLirfs.RData", compress=T)
rm(bestML.irfs, bestML.A0)
gc()


#########################################
# Flatprior model IRFs
#########################################

# Load posterior sample
load("flatpriorposterior.RData")

# Normalize signs of shocks and just use the first chain.
ident <- matrix(0, 9, 9)
ident[which(flatprior.A0$ident==1)] <- signlist
flatprior.A0$N2 <- N2

# MCMC Sample the IRFS
flatprior.irfs <- mc.irf(flatprior.model, nsteps=48,
                         A0.posterior=flatprior.A0,
                         sign.list=diag(ident))

# Plot the results
pdf(file="flatpriorirfs.pdf", width=9, height=6)
plot(100*flatprior.irfs, method="Sims-Zha2", probs=c(0.16, 0.84),
     varnames=vnames)
dev.off()

save(flatprior.irfs, file="flatpriorirfs.RData", compress=T)
rm(flatprior.irfs, flatprior.A0)
gc()

# Quit
q(save="no")
